part of ali_rtc_fluttify;

class _AndroidEventDelegate extends com_alivc_rtc_AliRtcEngineEventListener
    with _Callbacks {
  @override
  Future<void> onPublishResult(int result, String publishId) async {
    if (_onPublishResult != null) {
      _onPublishResult(result);
    }
  }

  @override
  Future<void> onUnpublishResult(int result) async {
    if (_onUnpublishResult != null) {
      _onUnpublishResult(result);
    }
  }

  @override
  Future<void> onNetworkQualityChanged(
    String uid,
    com_alivc_rtc_AliRtcEngine_AliRtcNetworkQuality upQuality,
    com_alivc_rtc_AliRtcEngine_AliRtcNetworkQuality downQuality,
  ) async {
    if (_onNetworkQualityChanged != null) {
      _onNetworkQualityChanged(
        uid,
        NetworkQuality.values[upQuality.index],
        NetworkQuality.values[downQuality.index],
      );
    }
  }

  @override
  Future<void> onUpdateRoleNotify(
    com_alivc_rtc_AliRtcEngine_AliRTCSDK_Client_Role oldRole,
    com_alivc_rtc_AliRtcEngine_AliRTCSDK_Client_Role newRole,
  ) async {
    if (_onUpdateRoleNotify != null) {
      _onUpdateRoleNotify(
        ClientRole.values[oldRole.index],
        ClientRole.values[newRole.index],
      );
    }
  }

  @override
  Future<void> onNetworkQualityProbeTest(
    com_alivc_rtc_AliRtcEngine_AliRtcNetworkQuality quality,
  ) async {
    if (_onNetworkQualityProbeTest != null) {
      _onNetworkQualityProbeTest(NetworkQuality.values[quality.index]);
    }
  }
}

class _AndroidNotifyDelegate extends com_alivc_rtc_AliRtcEngineNotify
    with _Callbacks {
  // @override
  // Future<void> onRemoteUserUnPublish(
  //   com_alivc_rtc_AliRtcEngine rtcEngine,
  //   String userId,
  // ) async {
  //   if (_onRemoteUserUnPublish != null) {
  //     _onRemoteUserUnPublish(userId);
  //   }
  // }

  @override
  Future<void> onRemoteUserOnLineNotify(String uid) async {
    if (_onRemoteUserOnLineNotify != null) {
      _onRemoteUserOnLineNotify(uid);
    }
  }

  @override
  Future<void> onRemoteUserOffLineNotify(String uid) async {
    if (_onRemoteUserOffLineNotify != null) {
      _onRemoteUserOffLineNotify(uid);
    }
  }

  @override
  Future<void> onRemoteTrackAvailableNotify(
    String uid,
    com_alivc_rtc_AliRtcEngine_AliRtcAudioTrack audioTrack,
    com_alivc_rtc_AliRtcEngine_AliRtcVideoTrack videoTrack,
  ) async {
    if (_onRemoteTrackAvailableNotify != null) {
      _onRemoteTrackAvailableNotify(
        uid,
        AudioTrack.values[audioTrack.index],
        VideoTrack.values[videoTrack.index],
      );
    }
  }
}

class _IOSDelegate extends NSObject with ios.AliRtcEngineDelegate, _Callbacks {
  @override
  Future<void> onPublishResult_isPublished(
    int result,
    bool isPublished,
  ) async {
    if (isPublished) {
      if (_onPublishResult != null) {
        _onPublishResult(result);
      }
    } else {
      if (_onUnpublishResult != null) {
        _onUnpublishResult(result);
      }
    }
  }

  @override
  Future<void> onRemoteUserOnLineNotify(String uid) async {
    if (_onRemoteUserOnLineNotify != null) {
      _onRemoteUserOnLineNotify(uid);
    }
  }

  @override
  Future<void> onRemoteUserOffLineNotify(String uid) async {
    if (_onRemoteUserOffLineNotify != null) {
      _onRemoteUserOffLineNotify(uid);
    }
  }

  @override
  Future<void> onNetworkQualityChanged_upNetworkQuality_downNetworkQuality(
    String uid,
    ios.AliRtcNetworkQuality upQuality,
    ios.AliRtcNetworkQuality downQuality,
  ) async {
    if (_onNetworkQualityChanged != null) {
      _onNetworkQualityChanged(
        uid,
        NetworkQuality.values[upQuality.index],
        NetworkQuality.values[downQuality.index],
      );
    }
  }

  @override
  Future<void> onRemoteTrackAvailableNotify_audioTrack_videoTrack(
    String uid,
    ios.AliRtcAudioTrack audioTrack,
    ios.AliRtcVideoTrack videoTrack,
  ) async {
    if (_onRemoteTrackAvailableNotify != null) {
      _onRemoteTrackAvailableNotify(
        uid,
        AudioTrack.values[audioTrack.index],
        VideoTrack.values[videoTrack.index],
      );
    }
  }

  @override
  Future<void> onUpdateRoleNotifyWithOldRole_newRole(
    ios.AliRtcClientRole oldRole,
    ios.AliRtcClientRole newRole,
  ) async {
    if (_onUpdateRoleNotify != null) {
      _onUpdateRoleNotify(
        ClientRole.values[oldRole.index],
        ClientRole.values[newRole.index],
      );
    }
  }

  @override
  Future<void> onLastmileDetectResultWithQuality(
    ios.AliRtcNetworkQuality quality,
  ) async {
    if (_onNetworkQualityProbeTest != null) {
      _onNetworkQualityProbeTest(NetworkQuality.values[quality.index]);
    }
  }
}

mixin _Callbacks {
  /// 网络测速回调
  void Function(NetworkQuality quality) _onNetworkQualityProbeTest;

  /// 角色更新回调
  void Function(ClientRole oldRole, ClientRole newRole) _onUpdateRoleNotify;

  /// 当远端用户的流发生变化时，返回这个消息
  ///
  /// audioTrack = AliRtcAudioTrackNo并且videoTrack = AliRtcVideoTrackNo时，代表远端用户停止推流。
  /// audioTrack或者videoTrack为非No的参数值时代表远端用户对应的推流状态。
  void Function(String uid, AudioTrack audioTrack, VideoTrack videoTrack)
      _onRemoteTrackAvailableNotify;
  void Function(String uid, NetworkQuality up, NetworkQuality down)
      _onNetworkQualityChanged;
  void Function(String uid) _onRemoteUserOffLineNotify;
  void Function(String uid) _onRemoteUserOnLineNotify;
  void Function(int result /*, String publishId */) _onPublishResult;
  void Function(int result) _onUnpublishResult;
}
